#ifndef LEVEL_HPP
#define LEVEL_HPP

#include <cstdio>

#include "global.hpp"
#include "conf.hpp"

// Contains initial state of all cells plus initial pos of the robot
// this is a static structure that does not change as the robot moves around
struct level
{
    // Initialize to empty level with the robot in the center facing up;
    level();

    // Reset to empty level with the robot in the center facing up
    void reset();

    // Possible cell states in a level
    struct cell {
        enum type {
            white, // there is a white button
            dot, // cell empty
            gray, // a gray button
            wall, // a wall
        };
    };

    // Initial content of every cell
    cell::type cells[conf::playground_unit_count_x]
        [conf::playground_unit_count_y];

    // Initial position of the robot
    int robot_x, robot_y;

    // Initial direction of the robot
    direction::type robot_dir;
};

// Additional/Dynamic information about a level resulting from the movement
// of the robot (e.g current pos/direction of the robot, pressed buttons,
// trail of the robot..etc)
struct dyn_level
{
    // Init to no info (no trail, no button pressed,...)
    dyn_level(direction::type dir=direction::up,
        int rx=conf::playground_unit_count_x/2,
        int ry=conf::playground_unit_count_y/2);

    // reset to no info (no trail, no button pressed) plus
    // given pos&dir for robot
    void reset(direction::type dir=direction::up,
        int rx=conf::playground_unit_count_x/2,
        int ry=conf::playground_unit_count_y/2);

    // Remove all pressed flags (e.g after the robot moves over a gray button)
    void unpress_all();

    // Direction the robot is currently facing
    direction::type robot_dir;

    // Current position of the robot
    int robot_x, robot_y;

    struct cell_info {
        // For each cell we hold a bitwise combination of these
        enum type {
            pressed=1<<1, // The button there is pressed
            trail_up=1<<2, // There is a trail up
            trail_down=1<<3, // trail down
            trail_right=1<<4, // trail right
            trail_left=1<<5 // trail left
        };
    };

    // A byte (bitwise of cell_info) per cell
    char cells[conf::playground_unit_count_x]
        [conf::playground_unit_count_y];

};

// Read a level out of a file
// returns true if everything went fine
bool read(level& lev, std::FILE* fp);

#endif //LEVEL_HPP

